/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details ( see the LICENSE file ).
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ludo.blocklist;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringReader;
import javax.swing.text.Document;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.parser.ParserDelegator;

/**
 * @author <a href="mailto:masterludo@gmx.net">Ludovic Kim-Xuan Galibert </a>
 * @revision $Id: SimpleHtmlToPlainParser.java,v 1.1 2004/12/08 22:57:51 masterludo Exp $ @created Mar 5, 2004
 */
public class SimpleHtmlToPlainParser extends HTMLEditorKit.ParserCallback {
	private boolean ignoreText;

	private boolean breakTag;

	private StringBuffer buffer;
	
	private static SimpleHtmlToPlainParser instance;
	
	/**
	 * 
	 * @return
	 */
	public static SimpleHtmlToPlainParser getInstance() {
		if (null == instance) {
			instance = new SimpleHtmlToPlainParser();
		}
		return instance;
	}
	
	
	protected SimpleHtmlToPlainParser() {
	}
	
	/**
	 * 
	 * @param aHtmlString
	 * @return
	 */
	public String parse(String aHtmlString) {
		String result = null;
		buffer = new StringBuffer(1024);
		Reader reader = new StringReader(aHtmlString);
		try {
			new ParserDelegator().parse(reader, this, true);
			result = buffer.toString();
			buffer = null;
		} catch (IOException ioe) {
			ioe.printStackTrace();
			HTMLEditorKit kit = new HTMLEditorKit();
			Document doc = kit.createDefaultDocument();
			try {
				kit.read(reader, doc, 0);
				result = doc.getText(0, doc.getLength()).trim();
			} catch (Exception ex) {
				// TODO
				result = "";
			}
		}
		try {
			reader.close();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		buffer = null;
		return result;
	}
	
	/**
	 * 
	 * @param aStream
	 * @return
	 */
	public String parse(InputStream aStream) {
		String result = null;
		buffer = new StringBuffer(1024);
		Reader reader = new InputStreamReader(aStream);
		try {
			new ParserDelegator().parse(reader, this, true);
			result = buffer.toString();
			buffer = null;
		} catch (IOException ioe) {
			ioe.printStackTrace();
			HTMLEditorKit kit = new HTMLEditorKit();
			Document doc = kit.createDefaultDocument();
			try {
				kit.read(reader, doc, 0);
				result = doc.getText(0, doc.getLength()).trim();
			} catch (Exception ex) {
				// TODO
				result = "";
			}
		}
		try {
			reader.close();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		buffer = null;
		return result;
	}

	/**
	 * @param data
	 * @param pos
	 */
	public void handleComment(char[] data, int pos) {
	}

	/**
	 * @param eol
	 */
	public void handleEndOfLineString(String eol) {
	}

	/**
	 * @param tag
	 * @param pos
	 */
	public void handleEndTag(HTML.Tag tag, int pos) {
		if (tag.equals(HTML.Tag.STYLE) || tag.equals(HTML.Tag.SCRIPT)) {
			ignoreText = false;
		}
		if (tag.equals(HTML.Tag.BR)) {
			breakTag = true;
		}
	}

	/**
	 * @param errorMsg
	 * @param pos
	 */
	public void handleError(String errorMsg, int pos) {
	}

	/**
	 * 
	 * @param tag
	 * @param a
	 * @param pos
	 */
	public void handleMutableTag(HTML.Tag tag, MutableAttributeSet a, int pos) {
	}

	/**
	 * @param tag
	 * @param a
	 * @param pos
	 */
	public void handleSimpleTag(HTML.Tag tag, MutableAttributeSet a, int pos) {
		if (tag.equals(HTML.Tag.BR)) {
			breakTag = true;
		}
	}

	/**
	 * @param tag
	 * @param a
	 * @param pos
	 */
	public void handleStartTag(HTML.Tag tag, MutableAttributeSet a, int pos) {
		if (tag.equals(HTML.Tag.STYLE) || tag.equals(HTML.Tag.SCRIPT)) {
			ignoreText = true;
		}
	}

	/**
	 * @param data
	 * @param pos
	 */
	public void handleText(char[] data, int pos) {
		if (!ignoreText) {
			if (breakTag) {
				buffer.append("\n");
				breakTag = false;
			}
			buffer.append(data);
		}
	}
}
