/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details ( see the LICENSE file ).
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ludo.logging;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Formatter;
import java.util.logging.LogRecord;

/** Simple formatter for a logger using following pattern:<br>
 * d MMM yyy HH:mm:ss - <logger name> -> <message>
 * @author  <a href="mailto:masterludo@gmx.net">Ludovic Kim-Xuan Galibert</a>
 * @revision $Id: MiniFormatter.java,v 1.1 2004/12/08 22:57:52 masterludo Exp $
 * @created 12 novembre 2003, 09:26
 */
public class MiniFormatter extends Formatter {
  
  /** Holds a date object */
  private Date _date = new Date();
  
  /** Holds the SimpleDateFormat */
  private static DateFormat _format = new SimpleDateFormat("d MMM yyy HH:mm:ss");
  
  /** Holds the system line separator string representation */
  private static final String LINE_SEPARATOR = System.getProperty("line.separator");
  
  /** Format the LogRecord using following pattern:<br>
   * d MMM yyy HH:mm:ss - <log level> -> <message>
   * @param aRecord  the LogRecord to format
   * @return  a formatted string representation of the LogRecord
   */
  public String format(LogRecord aRecord) {
    StringBuffer bf = new StringBuffer();
    _date.setTime(aRecord.getMillis());
    bf.append(_format.format(_date));
    bf.append(" - ");
    bf.append(aRecord.getLevel());
    //bf.append(aRecord.getLoggerName());
    bf.append(" -> ");
    bf.append(aRecord.getMessage());
    bf.append(LINE_SEPARATOR);
    return bf.toString();
  }
  
}
