/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details ( see the LICENSE file ).
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ludo.plugins.azureus;

import org.eclipse.swt.widgets.Composite;
import org.gudy.azureus2.plugins.PluginView;
import org.ludo.config.ConfigEntry;
import org.ludo.safepeer.ui.swt.SafePeerSwtGui;

import org.gudy.azureus2.ui.swt.plugins.*;

/**
 * PluginView for the SafePeer Azureus plugin.
 * @author  <a href="mailto:masterludo@gmx.net">Ludovic Kim-Xuan Galibert</a>
 * @revision $Id: AzureusSafePeerView.java,v 1.1 2004/12/08 22:57:50 masterludo Exp $
 * @created 26. January 2004, 12:35
 */
public class AzureusSafePeerView implements UISWTViewEventListener {
	
	/** Holds the ConfigEntry */
	private ConfigEntry configEntry;
	private Composite composite;
	
	/**
	 * Creates a new AzureusSafePeerView.
	 * @param anEntry  the CongigEntry to use for the configuration.
	 */
	public AzureusSafePeerView(ConfigEntry anEntry) {
		setConfigEntry(anEntry);
	}

	/**
	 * Initialize the PluginView by setting the main composite with the given parent.
	 * @param aParent  the parent composite of the main composite of this PluginView.
	 */
	private void initialize(Composite aParent) {
		setComposite( new SafePeerSwtGui(getConfigEntry(), aParent).getComposite() );
	}
	
	/**
	 * Gets the ConfigEntry to be used for the configuration.
	 * @return  a ConfigEntry object.
	 */
	protected ConfigEntry getConfigEntry() {
		return configEntry;
	}

	/**
	 * Sets the ConfigEntry to be used for the configuration.
	 * @param anEntry  a ConfigEntry object.
	 */
	protected void setConfigEntry(ConfigEntry anEntry) {
		configEntry = anEntry;
	}

	/**
	 * Sets the main composite of the PluginView.
	 * @param aComposite  a Composite, must not be null.
	 */
	protected void setComposite(Composite aComposite) {
		composite = aComposite;
	}

	  boolean isCreated = false;
	  public boolean eventOccurred(UISWTViewEvent event) {
	    switch (event.getType()) {
	      case UISWTViewEvent.TYPE_CREATE:
	        if (isCreated) // Comment this out if you want to allow multiple views!
	          return false;

	        isCreated = true;
	        break;

	      case UISWTViewEvent.TYPE_DESTROY:
	        isCreated = false;
	        break;

	      case UISWTViewEvent.TYPE_INITIALIZE:
	        initialize((Composite)event.getData());
	        break;

	    }

	    return true;
	  }
	
}
