/*
 * SafePeer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details ( see the LICENSE file ).
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ludo.safepeer.cache;

import java.util.List;
import org.ludo.net.IpRange;

/** The evil IPs cache interface.
 * @author  <a href="mailto:masterludo@gmx.net">Ludovic Kim-Xuan Galibert</a>
 * @revision $Id: EvilIPCache.java,v 1.1 2004/12/08 22:57:50 masterludo Exp $
 * @created 10. November 2003, 10:42
 */
public interface EvilIPCache {
  
  /** Add an EvilPeerEntry with given organization and lower and upper IP bounds
   * @param anOrganization  the organization or description corresponding to the IP range
   * @param aLowerIPBound the lower IP bound as string of the range
   * @param aGreaterIPBound the upper IP bound as string of the range
   */
  public void addEvilPeerEntry(String anOrganization, String aLowerIPBound, String aGreaterIPBound);
  
  /** Add the given EvilPeerEntry to the cache
   * @param anEntry  the EvilPeerEntry to add to the cache
   */
  public void addEvilPeerEntry(IpRange anEntry);
  
  /** Add the given list EvilPeerEntries to the cache
   * @param someEntries  the EvilPeerEntries to add to the cache
   */
  public void addEvilPeerEntry(List someEntries);
  
  /** Check if the given IP if is an evil IP known in the DB
   * @param anIp  the IP to check
   * @return  true if the IP is evil, otherwise false
   */
  public boolean isEvil(String anIp);
  
  /** Gets the list of all EvilPeerEntries in the cache
   * @return  the list of all EvilPeerEntries in the cache. It is recommanded to return an unmodifiable collection.
   */
  public List getEvilPeerEntries();
  
  /** Gets the cache of IPs that have already been blocked
   * @return  an unmodifiable list of IPs as string, cannot be null
   */
  public List retrieveBlockedCache();
  
  /** Clears the cache.
   */
  public void clear();
}
