/*
 * SafePeer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details ( see the LICENSE file ).
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ludo.safepeer.database;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import org.apache.commons.httpclient.HostConfiguration;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.GetMethod;
import org.ludo.blocklist.BlocklistManager;
import org.ludo.config.ConfigConstants;
import org.ludo.config.ConfigEntry;
import org.ludo.net.HttpConstants;
import org.ludo.util.ArchiveUtilities;
import org.ludo.util.HtmlInputStreamToFileParser;

/** Implementation of DatabaseDataRetriever using PeerGuardian's DB as source.
 * @author  <a href="mailto:masterludo@gmx.net">Ludovic Kim-Xuan Galibert</a>
 * @revision $Id: PeerGuardianDatabaseDataRetriever.java,v 1.2 2004/12/09 18:45:38 masterludo Exp $
 * @created 10. November 2003, 15:08
 */
public class PeerGuardianDatabaseDataRetriever implements DatabaseDataRetriever {
  
  /** Creates a new PeerGuardianDatabaseDataRetriever with the given config path.
   */
  public PeerGuardianDatabaseDataRetriever() {
  	/*
    if (parseConfig(aConfigEntry)) {
      if ( !retrieveData() ) {
        MiniLogger log = DefaultMiniLogger.getInstance(SafePeer.LOG_IDENTIFIER);
        log.warn("The data could not be retrieved from the database, trying with cache file");
        retrieveDataFromCache(aConfigEntry);
      }
    }
    */
  }
  
  /**
   * Parses the config file to configure the databases using the given ConfigEntry.
   * @param aConfigEntry  the ConfigEntry to use for the configuration.
   * @return  true if the config file could be parsed successfuly, otherwise false.
   */
  /*
  protected boolean parseConfig(ConfigEntry aConfigEntry) {
    boolean result = false;
    MiniLogger log = DefaultMiniLogger.getInstance(SafePeer.LOG_IDENTIFIER);
    try {
      String mainDb = aConfigEntry.getProperty(MAIN_DB_PROPERTY_KEY);
      log.info("Main DB is at: " + mainDb);
      setDatabaseURL( new URL(mainDb) );
      String secondaryDbs = aConfigEntry.getProperty(SECONDARY_DB_PROPERTY_KEY);
      StringTokenizer tk = new StringTokenizer(secondaryDbs, ",");
      List secondaryUrls = new ArrayList(tk.countTokens());
      while (tk.hasMoreTokens()) {
        String token = tk.nextToken();
        log.info("Secondary DB has been found: " + token);
        try {
          secondaryUrls.add( new URL(token) );
        } catch (Exception secex) {
          log.warn("Following secondary DB could not added: " + token);
          log.warn(secex.toString());
        }
      }
      setSecondaryDatabases(secondaryUrls);
      
      String connectTO = aConfigEntry.getProperty(SafepeerConfigConstants.SOCKET_CONNECT_TIMEOUT);
      String readTO = aConfigEntry.getProperty(SafepeerConfigConstants.SOCKET_READ_TIMEOUT);
      setConnectTimeout(Long.valueOf(connectTO).longValue() * 1000); // don't forget *1000 -> entries in seconds
      setReadTimeout(Long.valueOf(readTO).longValue() * 1000); // don't forget *1000 -> entries in seconds
      
      result = true;
    } catch (Exception ex) {
      result = false;
      log.error("The config file could not be loaded, no database URL can be retrieved");
      log.error(ex.toString());
    }
    return result;
  }
  */
  
  /** Retrieve the evil IPs database data.
   * @param anURL  the URL from which the data should be retrieved.
   * @param anEntry  a ConfigEntry, must not be <code>null</code>.
   * @return  a DatabaseData object, can be <code>null</code> if no
   * data could be retrieved.
   */
  /*
  public String retrieveData(URL anURL, ConfigEntry anEntry) {
    String data = null;
    //MiniLogger log = DefaultMiniLogger.getInstance(SafePeer.LOG_IDENTIFIER);
    try {
      data = URLParser.retrieveDataFromURL(anURL, anEntry);
      //log.info("Database data succesfuly retrieved");
    } catch (Exception ex) {
    	ex.printStackTrace();
    	// TODO
    }
    return data;
  }
  */
  
  /** Retrieve the evil IPs database data.
   * @param anUrl  the URL from which the data should be retrieved.
   * @param aConfigEntry  a ConfigEntry, must not be <code>null</code>.
   * @return  a DatabaseData object, can be <code>null</code> if no
   * data could be retrieved.
   */
  public List retrieveDataFiles(URL anUrl, ConfigEntry aConfigEntry) {
  	List retVal = null;
  	System.out.println("Retrieving data from: " + anUrl.toString());
  	if (anUrl.getProtocol().equalsIgnoreCase("http")) {
  		GetMethod method = new GetMethod(anUrl.toString());
  		method.setFollowRedirects(true);
  		
  		String host = anUrl.getHost();
  		method.addRequestHeader(HttpConstants.HEADER_HOST, host);
  		HttpClient client = new HttpClient();
  		HostConfiguration config = new HostConfiguration();
  		BlocklistManager.handleProxySettings(aConfigEntry, config);
  		config.setHost(host);
  		client.setHostConfiguration(config);
  		long timeout;
  		String toval = aConfigEntry.getProperty(ConfigConstants.SOCKET_CONNECT_TIMEOUT);
  		if (null != toval) {
  			timeout = Long.parseLong(toval) * 1000; // timeout is given in seconds
  		} else {
  			timeout = 60000;
  		}
  		client.getParams().setConnectionManagerTimeout(timeout);
  		try {
  			client.executeMethod(method);
  			if (200 == method.getStatusCode() || 301 == method.getStatusCode()
						|| 302 == method.getStatusCode()) {
					if (301 == method.getStatusCode() || 302 == method.getStatusCode()) { // (301|302) MOVED
						URL newUrl =
							new URL(method.getResponseHeader(HttpConstants.HEADER_LOCATION).getValue());
						retVal = retrieveDataFiles(newUrl, aConfigEntry);
					} else { // 200 OK
		  			String contentType =
		  				method.getResponseHeader(HttpConstants.HEADER_CONTENT_TYPE).getValue().toLowerCase();
		  			InputStream stream = null;
		  			if (contentType.startsWith("application/x-zip-compressed")
									|| contentType.startsWith("application/zip")) {
							File zip = ArchiveUtilities.downloadZipFromUrl(anUrl, "ranges");
							File unzipped = ArchiveUtilities.unzip(zip);
							stream = new FileInputStream(unzipped);
		  			} else if (contentType.startsWith("text/html")) {
		  				File plain = HtmlInputStreamToFileParser.getInstance().parse(
		  						method.getResponseBodyAsStream());
		  				stream = new FileInputStream(plain);
		  			} else if (contentType.startsWith("text/plain")) {
		  				stream = method.getResponseBodyAsStream();
		  			} else {
		  				System.out.println("Unsupported content-type: " + contentType);
		  			}
		  			if (null != stream) {
		  				retVal = writeToChunks(stream,
		  						anUrl.getPath().substring(anUrl.getPath().lastIndexOf("/")));
		  			}
					}
  			} else {
  				// TODO
  			}
  		} catch (Exception ex) {
  			ex.printStackTrace();
  		}
  		method.releaseConnection();
  	} else if (anUrl.getProtocol().equalsIgnoreCase("file")) {
  		try {
  			InputStream stream = null;
  			File file = new File(anUrl.getFile());
  			// TODO use datasource
  			if (file.getName().toLowerCase().endsWith(".zip")) {
					File zip = ArchiveUtilities.downloadZipFromUrl(anUrl, "ranges");
					File unzipped = ArchiveUtilities.unzip(zip);
					stream = new FileInputStream(unzipped);
  			} else if (file.getName().endsWith(".html") || file.getName().endsWith(".htm")) {
  				File plain = HtmlInputStreamToFileParser.getInstance().parse(
  						new FileInputStream(file));
  				stream = new FileInputStream(plain);
  			} else {
  				stream = new FileInputStream(file);
  			}/* else {
  				System.out.println("Unsupported content-type: " + contentType);
  			}*/
  			if (null != stream) {
  				retVal = writeToChunks(stream,
  						anUrl.getPath().substring(anUrl.getPath().lastIndexOf("/")));
  				stream.close();
  			}
  		} catch (Exception ex) {
  			ex.printStackTrace();
  		}
  	}
  	return retVal;
  }
  
  public static List writeToChunks(InputStream aStream, String aFileName) {
  	StringBuffer buffer = new StringBuffer(1200000);
  	List chunkList = new ArrayList(5);
		int currentSize = 0;
		
		try {
			File file = File.createTempFile(aFileName + "_", ".spchunk");
			FileOutputStream fos = new FileOutputStream(file);
			chunkList.add(file.getPath());
			
			Reader isReader = new InputStreamReader(aStream);
			BufferedReader reader = new BufferedReader(isReader);
			String line = null;
			while (null != (line = reader.readLine())) {
				currentSize += line.length();
				buffer.append(line);
				buffer.append(LINE_SEPARATOR);
				if (currentSize > 1000000) { // 1 Mb reached, make new chunk
					currentSize = 0;
					fos.write(buffer.toString().getBytes());
					fos.flush();
					fos.close();
					file = File.createTempFile(aFileName + "_", ".spchunk");
					chunkList.add(file.getPath());
					fos = new FileOutputStream(file);
					buffer = new StringBuffer(1200000);
				}
			}
			fos.write(buffer.toString().getBytes());
			buffer = null;
			fos.flush();
			fos.close();
			reader.close();
			isReader.close();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return chunkList;
  }
  
  public static final String LINE_SEPARATOR = System.getProperty("line.separator");
}
