package org.ludo.safepeer.ui.swt;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.ludo.blocklist.BlocklistConstants;
import org.ludo.config.ConfigConstants;
import org.ludo.config.ConfigEntry;

/**
 * @author  <a href="mailto:masterludo@gmx.net">Ludovic Kim-Xuan Galibert</a>
 * @revision $Id: BlocklistManagerSwtGui.java,v 1.3 2004/12/12 10:00:58 masterludo Exp $
 * @created Jul 8, 2004
 */
public class BlocklistManagerSwtGui implements SelectionListener {
	
	/** Holds the Text for the primary database Url */
	private Text blocklistUrlText;
	
	/** Holds the FileChooserField for the cache file */
	private FileChooserField cacheFileChooserField;
	
	/** Holds the FileChooserField for the log file */
	private FileChooserField logFileChooserField;
	
	/** Holds the Text for the socket connect timeout */
	private Text socketConnectTimeoutText;
	
	/** Holds the Text for the socket read timeout */
	private Text socketReadTimeoutText;
	
	/** Holds the Button for enabling the proxy */
	private Button enableProxyButton;
	
	/** Holds the Text for the proxy host */
	private Text proxyHostText;
	
	/** Holds the Text for the proxy port */
	private Text proxyPortText;
	
	/** Holds the save Button */
	private Button saveButton;
	
	/** Holds the composite */
	private Composite composite;
	
	/** Holds the ConfigEntry */
	private ConfigEntry configEntry;
	
	/** Holds the TabItem */
	private TabItem tabItem;

	/**
	 * 
	 * @param anEntry
	 * @param aFolder
	 */
	public BlocklistManagerSwtGui(ConfigEntry anEntry, TabFolder aFolder) {
		ConfigEntry entry =
			new ConfigEntry(anEntry.getConfigPath(),
					BlocklistConstants.CONFIG_FILE_NAME);
		setConfigEntry(entry);
		initGui(aFolder);
	}
	
	protected void initGui(TabFolder aFolder) {
		TabItem bmItem = new TabItem(aFolder, SWT.NULL);
		bmItem.setText("Blocklist");
		Composite bmComposite = new Composite(aFolder, SWT.NULL);
		bmItem.setControl(bmComposite);
		
		setComposite(bmComposite);
		GridLayout layout = new GridLayout();
		//3 columns, as first row will have 3 elements.
		layout.numColumns = 3;
		getComposite().setLayout(layout);

		initBlocklistUrl();
		initCacheFile();
		initLogFile();
		initSocketConnectTimeout();
		initSocketReadTimeout();
		initEnableProxy();
		initProxyHost();
		initProxyPort();
		
		initSaveButton();
		
		boolean isProxyEnabled = getEnableProxyButton().getSelection();
		getProxyHostText().setEnabled(isProxyEnabled);
		getProxyPortText().setEnabled(isProxyEnabled);
	}
	
	/**
	 * Initializes the label and text field for the configuration: "Blocklist URL".
	 */
	protected void initBlocklistUrl() {
		Label blocklistUrlLabel = new Label(getComposite(), SWT.NULL);
		blocklistUrlLabel.setText("Blocklist URL:");
		setBlocklistUrlText( new Text(getComposite(), SWT.BORDER) );
		getBlocklistUrlText().setText(
				getConfigEntry().getProperty(BlocklistConstants.BLOCKLIST_URL));
		GridData gridData =
			new GridData(
					GridData.VERTICAL_ALIGN_FILL | GridData.HORIZONTAL_ALIGN_FILL);
		gridData.widthHint = 400;
		gridData.horizontalSpan = 2;
		getBlocklistUrlText().setLayoutData(gridData);
	}
	
	/**
	 * Initializes the label and FileChooserField for the configuration: "Cache file".
	 */
	protected void initCacheFile() {
		Label cacheFileLabel = new Label(getComposite(), SWT.NULL);
		cacheFileLabel.setText("Cache file:");
		FileChooserField cacheFileField = new FileChooserField(getComposite());
		cacheFileField.getTextField().setText(
				getConfigEntry().getProperty(BlocklistConstants.BLOCKLIST_CACHE_FILE));
		setCacheFileChooserField(cacheFileField);
	}
	
	/**
	 * Initializes the label and FileChooserField for the configuration: "Log file".
	 */
	protected void initLogFile() {
		Label label4 = new Label(getComposite(), SWT.NULL);
		label4.setText("Log file:");
		FileChooserField logFileField = new FileChooserField(getComposite());
		logFileField.getTextField().setText(
				getConfigEntry().getProperty(BlocklistConstants.BLOCKLIST_LOG_FILE));
		setLogFileChooserField(logFileField);
	}
	
	/**
	 * Initializes the label and text field for the configuration: "Socket connect timeout".
	 */
	protected void initSocketConnectTimeout() {
		Label socketConnectLabel = new Label(getComposite(), SWT.NULL);
		socketConnectLabel.setText("Socket connect timeout:");
		
		setSocketConnectTimeoutText( new Text(getComposite(), SWT.BORDER) );
		getSocketConnectTimeoutText().setText(
				getConfigEntry().getProperty(BlocklistConstants.SOCKET_CONNECT_TIMEOUT));
		GridData gridData =
			new GridData(
					GridData.VERTICAL_ALIGN_FILL | GridData.HORIZONTAL_ALIGN_FILL);
		gridData.widthHint = 400;
		gridData.horizontalSpan = 2;
		getSocketConnectTimeoutText().setLayoutData(gridData);
	}
	
	/**
	 * Initializes the label and text field for the configuration: "Socket read timeout".
	 */
	protected void initSocketReadTimeout() {
		Label socketReadLabel = new Label(getComposite(), SWT.NULL);
		socketReadLabel.setText("Socket read timeout:");
		
		setSocketReadTimeoutText( new Text(getComposite(), SWT.BORDER) );
		getSocketReadTimeoutText().setText(
				getConfigEntry().getProperty(BlocklistConstants.SOCKET_READ_TIMEOUT));
		GridData gridData =
			new GridData(
					GridData.VERTICAL_ALIGN_FILL | GridData.HORIZONTAL_ALIGN_FILL);
		gridData.widthHint = 400;
		gridData.horizontalSpan = 2;
		getSocketReadTimeoutText().setLayoutData(gridData);
	}
	
	/**
	 * Initializes the label and button for the configuration: "Enable proxy".
	 */
	protected void initEnableProxy() {
		Label enableProxyLabel = new Label(getComposite(), SWT.NULL);
		enableProxyLabel.setText("Enable proxy:");
		setEnableProxyButton(new Button(getComposite(), SWT.CHECK));
		boolean selected = Boolean.valueOf(getConfigEntry().getProperty(ConfigConstants.ENABLE_HTTP_PROXY)).booleanValue();
		getEnableProxyButton().setSelection(selected);
		GridData gridData =
			new GridData(
					GridData.VERTICAL_ALIGN_FILL | GridData.HORIZONTAL_ALIGN_FILL);
		gridData.horizontalSpan = 2;
		getEnableProxyButton().setLayoutData(gridData);
		getEnableProxyButton().addSelectionListener(this);
	}
	
	/**
	 * Initializes the label and text field for the configuration: "Proxy host".
	 */
	protected void initProxyHost() {
		Label proxyHostLabel = new Label(getComposite(), SWT.NULL);
		proxyHostLabel.setText("Proxy host:");
		
		setProxyHostText(new Text(getComposite(), SWT.BORDER));
		String value = getConfigEntry().getProperty(ConfigConstants.HTTP_PROXY_HOST);
		if (null == value) value = "";
		getProxyHostText().setText(value);
		GridData gridData =
			new GridData(
					GridData.VERTICAL_ALIGN_FILL | GridData.HORIZONTAL_ALIGN_FILL);
		gridData.widthHint = 400;
		gridData.horizontalSpan = 2;
		getProxyHostText().setLayoutData(gridData);
	}
	
	/**
	 * Initializes the label and text field for the configuration: "Proxy port".
	 */
	protected void initProxyPort() {
		Label proxyPortLabel = new Label(getComposite(), SWT.NULL);
		proxyPortLabel.setText("Proxy host:");
		
		setProxyPortText(new Text(getComposite(), SWT.BORDER));
		String value = getConfigEntry().getProperty(ConfigConstants.HTTP_PROXY_PORT);
		if (null == value) value = "";
		getProxyPortText().setText(value);
		GridData gridData =
			new GridData(
					GridData.VERTICAL_ALIGN_FILL | GridData.HORIZONTAL_ALIGN_FILL);
		gridData.widthHint = 400;
		gridData.horizontalSpan = 2;
		getProxyPortText().setLayoutData(gridData);
	}
	
	/**
	 * Initializes the button for saving the configuration.
	 */
	protected void initSaveButton() {
		setSaveButton( new Button(getComposite(), SWT.BUTTON2) );
		getSaveButton().setText("Save");
		getSaveButton().addSelectionListener(this);
	}

	/**
	 * Saves the properties currently defined in the Gui into the config file.
	 */
	protected void saveProperties() {
		try {
			getConfigEntry().getProperties().put(BlocklistConstants.BLOCKLIST_URL,
					getBlocklistUrlText().getText());
			getConfigEntry().getProperties().put(BlocklistConstants.BLOCKLIST_CACHE_FILE,
					getCacheFileChooserField().getTextField().getText());
			getConfigEntry().getProperties().put(BlocklistConstants.BLOCKLIST_LOG_FILE,
					getLogFileChooserField().getTextField().getText());
			getConfigEntry().getProperties().put(ConfigConstants.SOCKET_CONNECT_TIMEOUT, getSocketConnectTimeoutText().getText());
			getConfigEntry().getProperties().put(ConfigConstants.SOCKET_READ_TIMEOUT, getSocketReadTimeoutText().getText());
			getConfigEntry().getProperties().put(ConfigConstants.ENABLE_HTTP_PROXY, Boolean.toString(getEnableProxyButton().getSelection()));
			getConfigEntry().getProperties().put(ConfigConstants.HTTP_PROXY_HOST, getProxyHostText().getText());
			getConfigEntry().getProperties().put(ConfigConstants.HTTP_PROXY_PORT, getProxyPortText().getText());
			
			// all keys found below from the existing properties file will be removed from the
			// list of missing keys, so that we can find out if a property is missing in the
			// file, maybe because it is from an older version
			List missingKeys = new ArrayList(getConfigEntry().getProperties().keySet());
			
			File configFile = new File(getConfigEntry().getConfigPath(), BlocklistConstants.CONFIG_FILE_NAME);
			StringBuffer buffer = new StringBuffer();
			BufferedReader reader = new BufferedReader(new FileReader(configFile));
			String line = null;
			while (null != (line = reader.readLine()) ) {
				if (line.indexOf("=") < 0) {
					buffer.append(line);
				} else {
					StringTokenizer tokenizer = new StringTokenizer(line, "=");
					String key = tokenizer.nextToken();
					if (getConfigEntry().getProperties().keySet().contains(key)) {
						missingKeys.remove(key); // key found, remove from missing keys
						addPropertyToBuffer(getConfigEntry(), buffer, key);
					} else {
						buffer.append(line);
					}
				}
				buffer.append(System.getProperty("line.separator"));
			}
			reader.close();
			
			// add the missing properties to the buffer
			for (Iterator iter = missingKeys.iterator(); iter.hasNext();) {
				addPropertyToBuffer(getConfigEntry(), buffer, iter.next().toString());
				buffer.append(System.getProperty("line.separator"));
			}
			
			FileWriter writer = new FileWriter(configFile);
			writer.write(buffer.toString());
			writer.flush();
			writer.close();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
	
	/**
	 * Adds the property with the given key to the buffer.
	 * @param aConfigEntry  the config entry holding the properties.
	 * @param aBuffer  the buffer.
	 * @param aKey  the key.
	 */
	protected void addPropertyToBuffer(ConfigEntry aConfigEntry, StringBuffer aBuffer, String aKey) {
		String value = getConfigEntry().getProperty(aKey);
		aBuffer.append(aKey);
		aBuffer.append("=");
		aBuffer.append(value);
	}
	
	/**
	 * Either saves the configuration or enables/disables all components, corresponding
	 * to the event that triggered this method.
	 * @param anEvent  the event that triggered this method, must not be null.
	 */
	public void widgetSelected(SelectionEvent anEvent) {
		if (null != anEvent && null != anEvent.getSource()) {
			if (anEvent.getSource().equals(getSaveButton())) {
				saveProperties();
			} else if (anEvent.getSource().equals(getEnableProxyButton())) {
				boolean isProxyEnabled = getEnableProxyButton().getSelection();
				getProxyHostText().setEnabled(isProxyEnabled);
				getProxyPortText().setEnabled(isProxyEnabled);
			}
		}
	}
	
	/**
	 * Does nothing yet, needed as class implements SelectionListener.
	 * @param anEvent  the event that triggered this method.  
	 */
	public void widgetDefaultSelected(SelectionEvent anEvent) {
		// Do nothing
	}
	
	/**
	 * Gets the text field for the configuration: "Blocklist URL".
	 * @return  a Text composite.
	 */
	protected Text getBlocklistUrlText() {
		return blocklistUrlText;
	}

	/**
	 * Sets the text field for the configuration: "Blocklist URL".
	 * @param aTextComposite  a Text composite.
	 */
	protected void setBlocklistUrlText(Text aTextComposite) {
		blocklistUrlText = aTextComposite;
	}
	
	/**
	 * Gets the FileChooserField for the cache file.
	 * @return  a FileChooserField.
	 */
	protected FileChooserField getCacheFileChooserField() {
		return cacheFileChooserField;
	}

	/**
	 * Sets the FileChooserField for the cache file.
	 * @param aFileChooserField  a FileChooserField.
	 */
	protected void setCacheFileChooserField(FileChooserField aFileChooserField) {
		cacheFileChooserField = aFileChooserField;
	}

	/**
	 * Gets the FileChooserField for the log file.
	 * @return  a FileChooserField.
	 */
	protected FileChooserField getLogFileChooserField() {
		return logFileChooserField;
	}

	/**
	 * Sets the FileChooserField for the cache file.
	 * @param aFileChooserField  a FileChooserField.
	 */
	protected void setLogFileChooserField(FileChooserField aFileChooserField) {
		logFileChooserField = aFileChooserField;
	}
	
	/**
	 * Gets the button for saving the configuration.
	 * @return  a Button composite.
	 */
	protected Button getSaveButton() {
		return saveButton;
	}

	/**
	 * Sets the button for saving the configuration.
	 * @param aButton  a Button composite.
	 */
	protected void setSaveButton(Button aButton) {
		saveButton = aButton;
	}
	
	/**
	 * @return Returns the socketConnectTimeoutText.
	 */
	protected Text getSocketConnectTimeoutText() {
		return socketConnectTimeoutText;
	}
	/**
	 * @param socketConnectTimeoutText The socketConnectTimeoutText to set.
	 */
	protected void setSocketConnectTimeoutText(Text socketConnectTimeoutText) {
		this.socketConnectTimeoutText = socketConnectTimeoutText;
	}
	/**
	 * @return Returns the socketReadTimeoutText.
	 */
	protected Text getSocketReadTimeoutText() {
		return socketReadTimeoutText;
	}
	/**
	 * @param socketReadTimeoutText The socketReadTimeoutText to set.
	 */
	protected void setSocketReadTimeoutText(Text socketReadTimeoutText) {
		this.socketReadTimeoutText = socketReadTimeoutText;
	}
	/**
	 * @return Returns the configEntry.
	 */
	protected ConfigEntry getConfigEntry() {
		return configEntry;
	}
	/**
	 * @param configEntry The configEntry to set.
	 */
	protected void setConfigEntry(ConfigEntry configEntry) {
		this.configEntry = configEntry;
	}
	/**
	 * @return Returns the composite.
	 */
	protected Composite getComposite() {
		return composite;
	}
	/**
	 * @param composite The composite to set.
	 */
	protected void setComposite(Composite composite) {
		this.composite = composite;
	}
	/**
	 * Gets the Text for the proxy host.
	 * @return  a Text.
	 */
	protected Text getProxyHostText() {
		return proxyHostText;
	}
	/**
	 * Sets the Text for the proxy host.
	 * @param aText a Text.
	 */
	protected void setProxyHostText(Text aText) {
		proxyHostText = aText;
	}
	/**
	 * Gets the Text for the proxy port.
	 * @return  a Text.
	 */
	protected Text getProxyPortText() {
		return proxyPortText;
	}
	/**
	 * Sets the Text for the proxy port.
	 * @param aText a Text.
	 */
	protected void setProxyPortText(Text aText) {
		proxyPortText = aText;
	}
	/**
	 * Gets the Button for enabling/disabling the proxy.
	 * @return  a Button.
	 */
	protected Button getEnableProxyButton() {
		return enableProxyButton;
	}
	/**
	 * Sets the Button for enabling/disabling the proxy.
	 * @param aButton  a Button.
	 */
	protected void setEnableProxyButton(Button aButton) {
		enableProxyButton = aButton;
	}
}
