/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details ( see the LICENSE file ).
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ludo.safepeer.ui.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Text;

/** Holder for a Text associated to a Button for opening a save dialog.
 * @author  <a href="mailto:masterludo@gmx.net">Ludovic Kim-Xuan Galibert</a>
 * @revision $Id: FileChooserField.java,v 1.1 2004/12/08 22:57:51 masterludo Exp $
 * @created 26. January 2004, 12:35
 */
public class FileChooserField implements SelectionListener {
	
	/** Holds the Text field */
	private Text textField;
	
	/** Holds the button */
	private Button button;
	
	/**
	 * Creates a new FileChooserField.
	 * @param aParent  the parent composite of this FileChooserField.
	 */
	public FileChooserField(Composite aParent) {
		initComponents(aParent);
		initListeners();
	}
	
	/**
	 * Initializes the components of this FileCooserField.
	 * @param aParent  the parent composite of this FileChooserField.
	 */
	protected void initComponents(Composite aParent) {
		setTextField(new Text(aParent, SWT.BORDER));
		GridData gridData = new GridData();
		gridData.widthHint = 300;
		getTextField().setLayoutData(gridData);
		
		setButton(new Button(aParent, SWT.BUTTON1));
		getButton().setText("...");
	}
	
	/**
	 * Initializes the listeners.
	 */
	protected void initListeners() {
		getButton().addSelectionListener(this);
	}
	
	
	/**
	 * Gets the button that pops up the file chooser.
	 * @return  a Button.
	 */
	public Button getButton() {
		return button;
	}

	/**
	 * Sets the button that pops up the file chooser.
	 * @param aButton  a Button.
	 */
	protected void setButton(Button aButton) {
		button = aButton;
	}

	/**
	 * @return Returns the textField.
	 */
	public Text getTextField() {
		return textField;
	}

	/**
	 * @param textField The textField to set.
	 */
	public void setTextField(Text textField) {
		this.textField = textField;
	}
	
	/**
	 * Pops up the file dialog for choosing the file. 
	 * @param anEvent  the event that triggered this method, must not be null.
	 */
	public void widgetSelected(SelectionEvent anEvent) {
		FileDialog dialog = new FileDialog(getButton().getShell(), SWT.SAVE);
		dialog.setText("Select file...");
		String filename = dialog.open();
		if (null != filename) {
			getTextField().setText(filename);
		}
	}
	
	/**
	 * Does nothing yet, needed as class implements SelectionListener.
	 * @param anEvent  the event that triggered this method.  
	 */
	public void widgetDefaultSelected(SelectionEvent anEvent) {
		// Do nothing
	}
	
	/**
	 * Enables/Disables the components.
	 * @param aFlag  true if components should be enabled, otherwise false.
	 */
	public void setEnabled(boolean aFlag) {
		getButton().setEnabled(aFlag);
		getTextField().setEnabled(aFlag);
	}

}

