package org.ludo.util;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import javax.swing.text.Document;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.parser.ParserDelegator;

/**
 * @author  <a href="mailto:masterludo@gmx.net">Ludovic Kim-Xuan Galibert</a>
 * @revision $Id: HtmlInputStreamToFileParser.java,v 1.1 2004/12/08 22:57:49 masterludo Exp $
 * @created Oct 30, 2004
 */
public class HtmlInputStreamToFileParser extends HTMLEditorKit.ParserCallback {
	
	private boolean ignoreText;

	private boolean breakTag;

	private FileOutputStream fos;
	
	private static HtmlInputStreamToFileParser instance;
	
	protected HtmlInputStreamToFileParser() {
	}
	
	public static HtmlInputStreamToFileParser getInstance() {
		if (null == instance) {
			instance = new HtmlInputStreamToFileParser();
		}
		return instance;
	}
	
	/**
	 * 
	 * @param aStream
	 * @param aFile
	 * @return
	 */
	public File parse(InputStream aStream) {
		File retVal = null;
		ignoreText = false;
		breakTag = false;
		Reader reader = new InputStreamReader(aStream);
		try {
			retVal = File.createTempFile("html2plain", ".sptmp");
			fos = new FileOutputStream(retVal);
			new ParserDelegator().parse(reader, this, true);
		} catch (IOException ioe) {
			ioe.printStackTrace();
			HTMLEditorKit kit = new HTMLEditorKit();
			Document doc = kit.createDefaultDocument();
			try {
				kit.read(reader, doc, 0);
				fos.write(doc.getText(0, doc.getLength()).trim().getBytes());
			} catch (Exception ex) {
				// TODO
			}
		}
		try {
			fos.flush();
			fos.close();
			reader.close();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		return retVal;
	}

	/**
	 * @param data
	 * @param pos
	 */
	public void handleComment(char[] data, int pos) {
	}

	/**
	 * @param eol
	 */
	public void handleEndOfLineString(String eol) {
	}

	/**
	 * @param tag
	 * @param pos
	 */
	public void handleEndTag(HTML.Tag tag, int pos) {
		if (tag.equals(HTML.Tag.STYLE) || tag.equals(HTML.Tag.SCRIPT)) {
			ignoreText = false;
		}
		if (tag.equals(HTML.Tag.BR)) {
			breakTag = true;
		}
	}

	/**
	 * @param errorMsg
	 * @param pos
	 */
	public void handleError(String errorMsg, int pos) {
	}

	/**
	 * 
	 * @param tag
	 * @param a
	 * @param pos
	 */
	public void handleMutableTag(HTML.Tag tag, MutableAttributeSet a, int pos) {
	}

	/**
	 * @param tag
	 * @param a
	 * @param pos
	 */
	public void handleSimpleTag(HTML.Tag tag, MutableAttributeSet a, int pos) {
		if (tag.equals(HTML.Tag.BR)) {
			breakTag = true;
		}
	}

	/**
	 * @param tag
	 * @param a
	 * @param pos
	 */
	public void handleStartTag(HTML.Tag tag, MutableAttributeSet a, int pos) {
		if (tag.equals(HTML.Tag.STYLE) || tag.equals(HTML.Tag.SCRIPT)) {
			ignoreText = true;
		}
	}

	/**
	 * @param data
	 * @param pos
	 */
	public void handleText(char[] data, int pos) {
		if (!ignoreText) {
			try {
				if (breakTag) {
					fos.write(LINE_FEED);
					breakTag = false;
				}
				fos.write(new String(data).getBytes());
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}
	}
	
	private static final byte[] LINE_FEED = "\n".getBytes();
	
}
